MODULE mo_interpolate

  ! The module mo_interpolate contains subroutines to interpolate
  ! data given on grid 1 to grid 2.
  !
  ! subroutines: interpolate_index  : initializes indices necessary
  !                                   for interpolation
  !
  !              interpolate_reset  : deallocates arrays set by 
  !                                   interpolate_index
  !
  ! functions  : interpolate_polynom: computes polynom interpolation
  !                                   for specified polynomial orders 1,3,5,...
  !
  !              interpolate_spline : computes cubic spline interpolation
  !
  ! The subroutine interpolate_index has to be called before the actual
  ! interpolation in order to initialize the private module variables:
  !
  ! grid1  : grid 1
  ! grid2  : grid 2
  !
  ! ndim1  : dimension of grid1
  ! ndim2  : dimension of grid2
  !
  ! k1     : index for grid1
  ! k2     : index for grid2
  ! k2first: index of first coordinate of grid2 in the range of grid1,
  !          =0 if grid1 and grid2 do not overlap
  ! k2last : index of last coordinate of grid2 in the range of grid1,
  !          =0 if grid1 and grid2 do not overlap
  ! k2i    : contains for each coordinate of grid2 in the range of grid1 the
  !          index of the nearest bigger coordinate of grid1,
  !          =0 for coordinates of grid2 which are outside of grid1,
  !          dimension is taken from actual array, must be equal to
  !          dimension of grid2
  !
  ! Author: Marco Giorgetta, Max-Planck-Institute for Meteorology
  !         October 1999

  USE mo_kind,ONLY : rs ! for REAL of selected precision

  IMPLICIT NONE

  PRIVATE

  ! Grids, dimensions and indices
  REAL(rs),ALLOCATABLE,DIMENSION(:) :: grid1,grid2
  INTEGER                           :: ndim1,ndim2
  INTEGER                           :: k1   ,k2

  ! Interpolation indices, see above
  INTEGER                           :: k2first
  INTEGER                           :: k2last
  INTEGER,ALLOCATABLE,DIMENSION(:)  :: k2i

  ! Interpolation subroutines
  PUBLIC :: interpolate_index
  PUBLIC :: interpolate_reset
  PUBLIC :: interpolate_polynom
  PUBLIC :: interpolate_spline

CONTAINS
  !------------------------------------------------------------------------------
  SUBROUTINE interpolate_index(coord1,coord2,i2first,i2last)

    ! This subroutine initializes the module variables which are
    ! used by the interpolation subroutines.
    !
    ! Input:
    ! ------
    ! coord1  : coordinates of grid 1
    ! coord2  : coordinates of grid 2
    !
    ! Output:
    ! -------
    ! i2first : index of first coordinate of grid 2 in the range of grid 1
    ! i2last  : index of last coordinate of grid 2 in the range of grid 1
    !
    ! Data on grid 1 may be interpolated to the index range [i2first,i2last]
    ! of grid 2.
    !
    ! Results stored in module variables
    ! ----------------------------------
    ! k2first: = i2first
    ! k2last : = i2last
    ! k2i    : contains for each coordinate of grid2 in the range of grid1 the
    !          index of the nearest bigger coordinate of grid1, zero otherwise
    !
    ! Author: Marco Giorgetta, Max-Planck-Institute for Meteorology, Hamburg
    !         October 1999

    IMPLICIT NONE

    REAL(rs),INTENT(in),DIMENSION(:) :: coord1,coord2
    INTEGER  ,INTENT(out)            :: i2first,i2last

    REAL(rs)                         :: g1sign
    REAL(rs),DIMENSION(SIZE(coord1)) :: g1
    REAL(rs),DIMENSION(SIZE(coord2)) :: g2

    ! 1) Allocate and initialize grids
    ! --------------------------------
    ndim1=SIZE(coord1)
    ALLOCATE(grid1(ndim1))
    grid1=coord1

    IF (ndim1==1) STOP 'grid1 must have more than 1 coordinate'

    ndim2=SIZE(coord2)
    ALLOCATE(grid2(ndim2))
    grid2=coord2

    ! 2) Reverse sign of both grids if grid1 is descending
    ! ----------------------------------------------------
    g1sign=SIGN(1._rs,grid1(ndim1)-grid1(1))
    g1=grid1*g1sign
    g2=grid2*g1sign

    ! 3) Test monotonicity of grids
    ! -----------------------------
    DO k1=2,ndim1
       IF (g1(k1)<=g1(k1-1)) &
            STOP 'grid 1 is not monotonic'
    ENDDO
    DO k2=2,ndim2
       IF (g2(k2)<=g2(k2-1)) &
            STOP 'grid 2 is not monotonic'
    ENDDO

    ! 4) Test if grid1 and grid 2 have a common range
    ! -----------------------------------------------
    IF (g1(ndim1)<g2(1).OR.g1(1)>g2(ndim2)) &
         STOP 'grid 2 has no overlap with grid 1'

    ! 5) Find index of first element of grid2
    !    which is in the range of grid1
    ! ---------------------------------------
    k2first=0
    k2=1
    DO
       IF (g2(k2)>=g1(1)) THEN
          k2first=k2
       ELSE
          k2=k2+1
       ENDIF
       IF (k2first/=0) EXIT
    ENDDO
    IF (g2(k2first)>g1(ndim1)) &
         STOP 'grid 2 has no coordinate in the range of grid 1'
    i2first=k2first

    ! 6) Find index of last element of grid2
    !    which is in the range of grid1
    ! --------------------------------------
    k2last=0
    k2=ndim2
    DO
       IF (g2(k2)<=g1(ndim1)) THEN
          k2last=k2
       ELSE
          k2=k2-1
       ENDIF
       IF (k2last/=0) EXIT
    ENDDO
    IF (g2(k2last)<g1(1)) &
         STOP 'grid 2 has no coordinate in the range of grid 1'
    i2last=k2last

    ! 7) Find indices k2i for elements k2first to k2last of grid2
    ! -----------------------------------------------------------
    ALLOCATE(k2i(ndim2))
    k2i=0
    k2=k2first
    k1=2
    DO
       IF (g2(k2)<=g1(k1)) THEN
          k2i(k2)=k1
          k2=k2+1
       ELSE
          k1=k1+1
       ENDIF
       IF (k2>k2last) EXIT
    ENDDO

  END SUBROUTINE interpolate_index
  !------------------------------------------------------------------------------
  SUBROUTINE interpolate_reset

    ! interpolate_index allocates arrays for later use by interpolate_polynom or
    ! interpolate_spline. interpolate_reset deallocates these arrays such that
    ! the interpolation_index may be called again with new grids.

    DEALLOCATE(grid1,grid2,k2i)

  END SUBROUTINE interpolate_reset
  !------------------------------------------------------------------------------
  FUNCTION interpolate_polynom(data1,nr,undef) RESULT(data2)

    ! Subroutine interpolate_polnom interpolates the values of array data1
    ! at positions of grid1 to positions of grid2. This is done for all
    ! coordinates of grid2 which are in the range of grid1. The interpolation
    ! is done by an interpolation polynom of order nl. If grid2 contains
    ! coordinates outside of grid1, then "undef" is returned for these
    ! coordinates.
    !
    ! nl is is the locally applied polynomial order depending on method im
    ! nc is the largest odd number <= nr
    ! nr is the requested polynomial order.
    !
    ! Near the boundary the polynomial order nl is set as specified by the
    ! method selector im:
    !
    !   im=1 : maximum nl<=nc with interpolation point in centre 
    !          interval of polynomial
    !   im=2 : nl is increased by 1 if nl(im=1)<nc
    !   im=3 : nl is constant, nl=nc
    !
    ! For points of grid2 which are in the first or last interval of grid1
    ! the  interpolated values must stay in the range of the points of
    ! grid1 delimiting these intervals.
    !
    ! This subroutine uses the grids and the index information from the
    ! module variables, see comment there.
    !
    ! Input:
    ! ------
    ! data1  = data at coordinates of grid1
    ! nr     = requested polynomial order
    ! undef  = value for initialization of result array
    !
    ! Output:
    ! -------
    ! data2  = result array containing the interpolated values
    !          in the index range k2first to k2last, 0. elsewhere
    !
    ! Author: Marco Giorgetta, Max-Planck-Institute for Meteorology, Hamburg
    !         January 2000

    IMPLICIT NONE

    REAL(rs),INTENT(in), DIMENSION(:) :: data1 ! values on grid 1
    INTEGER ,INTENT(in)               :: nr    ! requested order of polynomial
    REAL(rs),INTENT(in), OPTIONAL     :: undef ! undefined value
    REAL(rs),DIMENSION(ndim2)         :: data2 ! values interpolated to grid 2

    INTEGER,PARAMETER                 :: im=1  ! method selector

    INTEGER :: nc   ! corrected order of polynomial (1<=nc<=nr and nc is odd)
    INTEGER :: nl   ! local order of polynomial     (1<=nl<=nc and nl is odd)

    INTEGER :: k10  ! grid1 index of first point in local range [0,nl]
    INTEGER :: k1nl ! grid1 index of last point in local range [0,nl]

    INTEGER  :: j1                     ! work variable
    REAL(rs) :: sum, mult              ! work variables for interpolation
    REAL(rs) :: max1, min1, maxn, minn ! work variqbles for boundaries

    ! 1) Test dimensions
    ! ------------------
    IF (SIZE(data1)/=ndim1) &
         STOP 'dimension of data1 is different from dimension of grid1'
    IF (SIZE(data2)/=ndim2) &
         STOP 'dimension of data2 is different from dimension of grid2'


    ! 2) Test requested polynomial order and correct if necessary
    ! -----------------------------------------------------------
    ! test requested polynomial order and correct if necessary
    ! order nc must be odd and 1 <= nc <= number of points of grid1 minus 1
    nc=MIN( MAX(1,1+((nr-1)/2)*2) , 1+(((ndim1-1)-1)/2)*2 )
    IF (nr/=nc) PRINT *,'order of polynomial changed from ',nr,' to ',nc


    ! 3) Initialize data2 array
    ! -------------------------
    IF (PRESENT(undef)) THEN
       data2=undef
    ELSE
       data2=0.
    END IF


    ! 4) compute interpolation polynomial by Lagrange formula
    ! -------------------------------------------------------
    DO k2=k2first,k2last

       SELECT CASE(im)
       CASE(1)
          ! Method: near boundary: centred order
          nl=MIN((k2i(k2)-1)*2-1,nc,(ndim1+1-k2i(k2))*2-1)
          k10=k2i(k2)-(nl+1)/2
          k1nl=k10+nl
       CASE(2)
          ! Method: near boundary: centred order + 1
          nl=MIN((k2i(k2)-1)*2,nc,(ndim1+1-k2i(k2))*2)
          k10=MIN(k2i(k2)-(nl+1)/2,ndim1-nl)
          k1nl=k10+nl
       CASE(3)
          ! Method: near boundary: constant order
          nl=nc
          k10=MAX(1,MIN(ndim1-nl,k2i(k2)-(nl+1)/2))
          k1nl=k10+nl
       END SELECT

       sum=0
       DO k1=k10,k1nl
          mult=data1(k1)
          DO j1=k10,k1-1
             mult=mult*(grid2(k2)-grid1(j1))/(grid1(k1)-grid1(j1))
          ENDDO
          DO j1=k1+1,k1nl
             mult=mult*(grid2(k2)-grid1(j1))/(grid1(k1)-grid1(j1))
          ENDDO
          sum=sum+mult
       ENDDO
       data2(k2)=sum
    ENDDO

    ! 5) special treatment for points of grid2 which are in first
    !    or last interval of grid 1.
    ! -----------------------------------------------------------
    ! The interpolated values for these points must stay in the
    ! range of the points of grid 1 delimiting these intervals.
    !
    ! a) interval between points 1 and 2 of grid 1
    max1=max(data1(1),data1(2))
    min1=min(data1(1),data1(2))
    where (k2i==2) data2=min(max1,max(data2,min1))
    !
    ! b) interval  between points ndim1-1 and ndim1 of grid 1
    maxn=max(data1(ndim1-1),data1(ndim1))
    minn=min(data1(ndim1-1),data1(ndim1))
    where (k2i==ndim1) data2=min(maxn,max(data2,minn))

  END FUNCTION interpolate_polynom
  !------------------------------------------------------------------------------
  FUNCTION interpolate_spline(data1,undef) RESULT(data2)

    ! Subroutine interpolate_spline interpolates the values of array data1
    ! at positions of grid1 to positions of grid2. This is done for all
    ! coordinates of grid2 which are in the range of grid1. The interpolation
    ! is done by a spline interpolation based on 3rd order piecewise polynomials.
    ! If grid2 contains coordinates outside of grid1, then zero is returned
    ! for these coordinates.
    !
    ! This subroutine uses the grids and the index information from the
    ! module variables, see comment there.
    !
    ! Input:
    ! ------
    ! data1  = data at coordinates of grid 1
    !
    ! Output:
    ! -------
    ! data2  = result array containing the interpolated values
    !          in the index range k2first to k2last, 0. elsewhere
    !
    ! Author: Marco Giorgetta, Max-Planck-Institute for Meteorology, Hamburg
    !         October 1999

    IMPLICIT NONE

    REAL(rs),INTENT(in), DIMENSION(:) :: data1 ! values on grid 1
    REAL(rs),INTENT(in), OPTIONAL     :: undef ! undefined value
    REAL(rs),DIMENSION(ndim2)         :: data2 ! values interpolated to grid 2

    REAL(rs),DIMENSION(SIZE(data1)-1) :: h     ! coordinate increments
    ! h(i)=xt(i+1)-xt(i)
    REAL(rs),DIMENSION(SIZE(data1)-1) :: c     ! co-diagonal in symmetric
    ! tridiagonal matrix A
    REAL(rs),DIMENSION(SIZE(data1))   :: d     ! diagonal of matrix A
    REAL(rs),DIMENSION(SIZE(data1))   :: b,s   ! vectors, As=b

    ! 1) Test dimensions
    ! ------------------
    IF (SIZE(data1)/=ndim1) &
         STOP 'dimension of data1 is different from dimension of grid1'
    IF (SIZE(data2)/=ndim2) &
         STOP 'dimension of data2 is different from dimension of grid2'

    ! 2) Initialize data2 array
    ! -------------------------
    IF (PRESENT(undef)) THEN
       data2=undef
    ELSE
       data2=0.
    END IF

    ! 3) Prepare for spline interpolation
    ! -----------------------------------
    ! The interpolation is done by a natural spline interpolant g(x).

    ! start of spline procedure
    ! Compute derivative s(x) of natural spline interpolant
    !
    ! h = distance between given points
    DO k1=1,ndim1-1
       h(k1)=grid1(k1+1)-grid1(k1)
    ENDDO
    !
    ! c = co-diagonal elements of matrix A of spline problem
    c=1/h
    !
    ! d = diagonal elements of A
    ! b = result vector in As=b
    k1=1
    d(k1)=2*c(k1)
    b(k1)=3*(data1(k1+1)-data1(k1))/h(k1)**2
    DO k1=2,ndim1-1
       d(k1)=2*(c(k1-1)+c(k1))
       b(k1)=3*((data1(k1)-data1(k1-1))/h(k1-1)**2+(data1(k1+1)-data1(k1))/h(k1)**2)
    ENDDO
    k1=ndim1
    d(k1)=2*c(k1-1)
    b(k1)=3*(data1(k1)-data1(k1-1))/h(k1-1)**2
    !
    ! s = vector s in As=b
    s=dydxsp(c,d,b)
    !
    ! end of spline procedure

    ! 4) Interpolate data1 on grid1 to grid2 for all elements of grid2
    !    which are in the range of grid1
    ! ----------------------------------------------------------------
    DO k2=k2first,k2last
       k1=k2i(k2)
       data2(k2)=( (grid2(k2)-grid1(k1))**2*data1(k1-1)+(grid2(k2)-grid1(k1-1))**2*data1(k1)   &
            +(grid2(k2)-grid1(k1-1))*(grid2(k2)-grid1(k1))**2*(s(k1-1)+2/h(k1-1)*data1(k1-1))  &
            +(grid2(k2)-grid1(k1-1))**2*(grid2(k2)-grid1(k1))*(s(k1)-2/h(k1-1)*data1(k1))    ) &
            /h(k1-1)**2
    ENDDO

  END FUNCTION interpolate_spline
  !------------------------------------------------------------------------------
  FUNCTION dydxsp(c,d,b)

    ! This subroutine solves the Matrix problem Ax=b. This is done in a LR 
    ! decomposition of A followed by forward and backward substitution.
    ! 
    ! A is symmetric, tridiagonal and positive definit. The elements of A
    ! are weights depending on the grid point distances.
    !
    ! Diagonal and codiagonal values of A are given in the arrays d and c.
    !
    ! x is the vector containing the first derivatives of the spline.
    !
    ! b contains weighted function value differences.
    !
    ! Author: Marco Giorgetta, Max-Planck-Institute for Meteorology
    !         October 1999


    IMPLICIT NONE

    REAL(rs),INTENT(in),DIMENSION(:) :: c      ! codiagonal elements
    REAL(rs),INTENT(in),DIMENSION(:) :: d      ! diagonal elements
    REAL(rs),INTENT(in),DIMENSION(:) :: b      ! right hand side vector
    REAL(rs),DIMENSION(SIZE(b))      :: dydxsp ! resulting derivatives

    REAL(rs)                         :: dd     ! work real
    REAL(rs),DIMENSION(SIZE(b))      :: cc     ! work array
    REAL(rs),DIMENSION(SIZE(b))      :: yy     ! work array

    INTEGER :: n      ! vector length
    INTEGER :: i      ! loop index

    n=SIZE(b)

    ! forward substitution -> Rx=yy, R = right triangular matrix with
    ! diag =1, codiag = cc(2:n), and all other elements = 0
    dd=d(1)
    yy(1)=b(1)/dd
    DO i=2,n
       cc(i)=c(i-1)/dd
       dd=d(i)-c(i-1)*cc(i)
       yy(i)=(b(i)-c(i-1)*yy(i-1))/dd
    ENDDO
    ! backward substitution
    DO i=n-1,1,-1
       yy(i)=yy(i)-cc(i+1)*yy(i+1)
    ENDDO

    dydxsp=yy

  END FUNCTION dydxsp
  !------------------------------------------------------------------------------
END MODULE mo_interpolate
