PROGRAM u_profile_extended_1984

  ! - Reads monthly mean profiles of zonal wind from three equatorial
  !   stations:
  !      Canton Island, 02 46 S / 171 43 W, Jan.1953-Aug.1967
  !      Gan/Maledives, 00 41 S /  73 09 E, Sep.1967-Dec.1975
  !      Singapore,     01 22 N / 103 55 E, Jan.1976 onwards
  !   interpolated to a high resolution vertical grid.
  !
  !   The data are read from the file u_profile_highres_195301-201210.txt.
  !   January to April 1984 are blended with January to April 2012 such that
  !   a smooth transition from Dec.2011 to the modified Jan.1984 is obtained.
  !
  !
  !=============================================================================

  IMPLICIT NONE

  ! Grids
  ! -----
  !   high resolution:
  INTEGER,  PARAMETER   :: nhighres=14
  REAL, PARAMETER       :: phighres(nhighres)=&
       & (/          10,12,15,20,25,30,35,40,45,50,60,70,80,90/)

  ! Observations of zonal wind
  ! --------------------------
  ! January 1953 to August 1967: Canton Island
  ! 15-70 hPa in Jan.53-Dec.55
  ! 10-70 hPa in Jan.56-Aug.67
  INTEGER, PARAMETER    :: nmonth1a=(1955-1952)*12
  INTEGER, PARAMETER    :: nmonth1b=(1966-1955)*12+8
  INTEGER, PARAMETER    :: nmonth1=nmonth1a+nmonth1b

  ! September 1967 to December 1975: Gan/Maledives
  INTEGER, PARAMETER    :: nmonth2=4+(1975-1967)*12

  ! January 1976 to December 1986: Singapore
  INTEGER, PARAMETER    :: nmonth3=(1986-1975)*12

  ! January 1987 to April 2012: Singapore
  INTEGER, PARAMETER    :: overlap=4
  INTEGER, PARAMETER    :: nmonth4=(2011-1986)*12+overlap

  ! time grid for full time of observed data
  ! ----------------------------------------
  INTEGER, PARAMETER    :: nmonth=nmonth1+nmonth2+nmonth3+nmonth4

  ! time grid for full time of observed data
  ! ----------------------------------------
  INTEGER, PARAMETER    :: istartyear=1953
  INTEGER, PARAMETER    :: inewstartyear=1984
  INTEGER, PARAMETER    :: idmonth=(inewstartyear-istartyear)*12
  INTEGER, PARAMETER    :: nmonthnew=nmonth-idmonth

  ! old and new, extended wind profiles
  ! -----------------------------------
  REAL                  :: uhigh(nhighres,nmonth)
  REAL                  :: uhighnew(nhighres,nmonthnew)

  ! value for missing wind observation
  ! ----------------------------------
  REAL, PARAMETER       :: undef=99.9

  ! Auxiliary variables
  ! -------------------
  INTEGER :: i,j,idate,nout


  !=============================================================================

  ! Read data from files 
  ! ---------------------
  ! write high resolution profiles to text data file
  OPEN(unit=21,file='u_profile_highres_195301-201210.txt',form='formatted')
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  READ(21,*)
  DO i=1,nmonth
     READ(21,'(i6,14f7.2)') idate,(uhigh(j,i),j=1,14)
  END DO
  CLOSE(21)

  !=============================================================================

  ! Construct modified data
  ! -----------------------

  ! 1. copy old data
  uhighnew(:,1:)=uhigh(:,idmonth+1:)

  ! 2. linear weighted overlap
  WHERE (uhighnew(:,1)/=undef.AND.uhigh(:,nmonth+1-4)/=undef) &
       & uhighnew(:,1)= ( 1.*uhighnew(:,1) + 4.*uhigh(:,nmonth+1-4) )/5.
  WHERE (uhighnew(:,2)/=undef.AND.uhigh(:,nmonth+1-3)/=undef) &
       & uhighnew(:,2)= ( 2.*uhighnew(:,2) + 3.*uhigh(:,nmonth+1-3) )/5.
  WHERE (uhighnew(:,3)/=undef.AND.uhigh(:,nmonth+1-2)/=undef) &
       & uhighnew(:,3)= ( 3.*uhighnew(:,3) + 2.*uhigh(:,nmonth+1-2) )/5.
  WHERE (uhighnew(:,4)/=undef.AND.uhigh(:,nmonth+1-1)/=undef) &
       & uhighnew(:,4)= ( 4.*uhighnew(:,4) + 1.*uhigh(:,nmonth+1-1) )/5.          

  !=============================================================================

  ! write extended high resolution profiles to GrADS data file
  OPEN(unit=21,file='u_profile_highres_201112_198401-200612',form='unformatted')
  nout = nmonthnew - (2011 - 2006) * 12 - overlap
  DO i=1,nout
     DO j=1,nhighres
        WRITE(21) uhighnew(j,i) ! copy to default precision REAL variable
     END DO
  END DO
  CLOSE(21)

  OPEN(unit=21,file='u_profile_highres_201112_198401-200612.txt',form='formatted')
  
  DO i=1,nout
     idate=(inewstartyear+(i-1)/12)*100+(i-((i-1)/12)*12)
     WRITE(21,'(i6,19f7.2)') idate,(uhighnew(j,i),j=1,14)
  END DO
  CLOSE(21)


  !=============================================================================

END PROGRAM u_profile_extended_1984
